local TreasureChestLogic = require("treasurechest.TreasureChestLogic")

-- 请求宝箱界面信息
function CG_TEEASURECHEST_QUERY(human, msg)
    TreasureChestLogic.TreasureChestLogic_Query(human)
end

-- 请求宝箱内奖励信息
function CG_TEEASURECHEST_PRIZE_QUERY(human, msg)
    TreasureChestLogic.TreasureChestLogic_QueryPrize(human, msg.nBoxType)
end

-- 请求打开宝箱
function CG_TEEASURECHEST_OPEN(human, msg)
    TreasureChestLogic.TreasureChestLogic_Open(human, msg.nBoxType, msg.nNum)
end

-- 请求自动打开宝箱
function CG_TEEASURECHEST_AUTO_OPEN(human, msg)
    TreasureChestLogic.TreasureChestLogic_AutoOpen(human, msg.nBoxType)
end

-- 请求领取积分奖励
function CG_TEEASURECHEST_GET_POINT_PRIZE(human, msg)
    TreasureChestLogic.TreasureChestLogic_GetPointPrize(human, msg.nID)
end


-- 请求领取积分奖励
function CG_TEEASURECHEST_ONETOUH_OPEN(human, msg)
    TreasureChestLogic.TreasureChestLogic_OneTouchOpen(human)
end

function CG_GET_ALL_GJBOX_NUM(human,  msg)
    TreasureChestLogic.TreasureChestLogic_GetAllBoxNum(human)
end